Policy

Trait Policy 

pub trait Policy<B, E> {
    // Required method
    fn redirect(&mut self, attempt: &Attempt<'_>) -> Result<Action, E>;

    // Provided methods
    fn on_request(&mut self, _request: &mut Request<B>) { ... }
    fn clone_body(&self, _body: &B) -> Option<B> { ... }
}
Trait for the policy on handling redirection responses.

Example

Detecting a cyclic redirection:

use http::{Method, Request, Uri};
use std::collections::HashSet;
use tower_http::follow_redirect::policy::{Action, Attempt, Policy};

#[derive(Clone)]
pub struct DetectCycle {
    uris: HashSet<(Method, Uri)>,
}

impl<B, E> Policy<B, E> for DetectCycle {
    fn redirect(&mut self, attempt: &Attempt<'_>) -> Result<Action, E> {
        if self.uris.contains(&(attempt.method().clone(), attempt.location().clone())) {
            Ok(Action::Stop)
        } else {
            self.uris.insert((attempt.previous_method().clone(), attempt.previous().clone()));
            Ok(Action::Follow)
        }
    }
}

Required Methods

fn redirect(&mut self, attempt: &Attempt<'_>) -> Result<Action, E>

Invoked when the service received a response with a redirection status code (3xx).

This method returns an Action which indicates whether the service should follow the redirection.

Provided Methods

fn on_request(&mut self, _request: &mut Request<B>)

Invoked right before the service makes a request, regardless of whether it is redirected or not.

This can for example be used to remove sensitive headers from the request or prepare the request in other ways.

The default implementation does nothing.

fn clone_body(&self, _body: &B) -> Option<B>

Try to clone a request body before the service makes a redirected request.

If the request body cannot be cloned, return None.

This is not invoked when B::size_hint returns zero, in which case B::default() will be used to create a new request body.

The default implementation returns None.

Dyn Compatibility

This trait is dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl<B, E, P> Policy<B, E> for &mut P
where P: Policy<B, E> + ?Sized,

Source

fn redirect(&mut self, attempt: &Attempt<'_>) -> Result<Action, E>

Source

fn on_request(&mut self, request: &mut Request<B>)

Source

fn clone_body(&self, body: &B) -> Option<B>

Source

impl<B, E, P> Policy<B, E> for Box<P>
where P: Policy<B, E> + ?Sized,

Source

fn redirect(&mut self, attempt: &Attempt<'_>) -> Result<Action, E>

Source

fn on_request(&mut self, request: &mut Request<B>)

Source

fn clone_body(&self, body: &B) -> Option<B>

Source

impl<B, E> Policy<B, E> for Result<Action, E>
where E: Clone,

Source

fn redirect(&mut self, _: &Attempt<'_>) -> Result<Action, E>

Implementors

Source

impl<B, E, F> Policy<B, E> for RedirectFn<F>
where F: FnMut(&Attempt<'_>) -> Result<Action, E>,

Source

impl<B, E> Policy<B, E> for Action

Source

impl<B, E> Policy<B, E> for FilterCredentials

Source

impl<B, E> Policy<B, E> for Limited

Source

impl<B, E> Policy<B, E> for SameOrigin

Source

impl<Bd, E, A, B> Policy<Bd, E> for And<A, B>
where A: Policy<Bd, E>, B: Policy<Bd, E>,

Source

impl<Bd, E, A, B> Policy<Bd, E> for Or<A, B>
where A: Policy<Bd, E>, B: Policy<Bd, E>,

Source

impl<F, B, E> Policy<B, E> for CloneBodyFn<F>
where F: Fn(&B) -> Option<B>,